@@ -1,9 +1,9 @@
 import copy
 import json
 from decimal import Decimal
+from typing import Optional
 
 import boto3
-
 from boto3.dynamodb.types import TypeSerializer, TypeDeserializer
 from botocore import exceptions
 
@@ -11,10 +11,15 @@ from botocore import exceptions
 class InsufficientArgumentsException(Exception):
     pass
 
+
 class IndexNotValidException(Exception):
     pass
 
 
+class InvalidArgumentsException(Exception):
+    pass
+
+
 class ConditionalCheckFailedException(Exception):
     pass
 
@@ -23,7 +28,7 @@ class DynamoDBDataStore(object):
     serializer = TypeSerializer()
     deserializer = TypeDeserializer()
 
-    def __init__(self, table_name, hash_key='PK', sort_key='SK', use_default_index_keys=True,  endpoint_url=None):
+    def __init__(self, table_name, hash_key='PK', sort_key='SK', use_default_index_keys=True, endpoint_url=None):
 
         DB_SETTINGS = {
             'TableName': table_name,
@@ -46,9 +51,9 @@ class DynamoDBDataStore(object):
                 'GSI2SK',
             ])
 
-        self.client = boto3.client('dynamodb',  endpoint_url=endpoint_url)
+        self.client = boto3.client('dynamodb', endpoint_url=endpoint_url)
 
-    def add_index_keys(self, keys_to_add:list):
+    def add_index_keys(self, keys_to_add: list):
         if type(keys_to_add) != list:
             raise TypeError('Expected a list but received ', type(keys_to_add))
         for key in keys_to_add:
@@ -147,10 +152,13 @@ class DynamoDBDataStore(object):
     def deserialize(self, document):
         return DynamoDBDataStore.deserializer.deserialize({'M': document})
 
-    def paginate(self, parameters):
+    def paginate(self, parameters: dict = None, paginator_type='query'):
+        if not parameters:
+            parameters = {}
+
         parameters['TableName'] = self.table_name
 
-        paginator = self.client.get_paginator('query')
+        paginator = self.client.get_paginator(paginator_type)
         return paginator.paginate(**parameters)
 
     def update_document(self, index=None, parameters=None):
@@ -214,24 +222,46 @@ class DynamoDBDataStore(object):
             self.table_name: request_items
         })
 
-    def get_documents(self, query=None, return_index=False):
-        if not query:
+    def get_documents(self, query: dict = None, return_index: bool = False, scan: bool = False) -> Optional[list]:
+        """
+        Perform a SCAN or a QUERY on a DynamoDB Table by setting the scan flag or providing query parameters.
+        Optionally return the index keys by setting the return_index flag.
+
+        :param query: A query dict containing DynamoDB query parameters
+        :type query: dict
+
+        :param return_index: A flag indicating if the indexes for the document should be returned with the document
+        :type return_index: bool
+
+        :param scan: A flag to indicate that the table should be scanned. cannot be used in conjunction with query
+        :type scan: bool
+
+        :return: The documents returned as a list of dicts or None if no documents were returned or in case of an error
+        """
+
+        if not query and not scan:
             raise InsufficientArgumentsException('You must provide a query')
+        if query and scan:
+            raise InvalidArgumentsException('You cannot specify a query when performing a scan. '
+                                            'Set scan to False if you wish to perform a query '
+                                            'or leave out the query parameter to perform a scan')
+
         try:
-            pages = self.paginate(query)
+            paginator_type = 'scan' if scan else 'query'
+            pages = self.paginate(query, paginator_type)
             documents = []
-    
+
             for page in pages:
                 items = page['Items']
-    
+
                 for item in items:
                     document = self.deserialize(item)
                     if not return_index:
                         for key in self.index_keys:
                             document.pop(key, None)
-    
+
                     documents.append(document)
-    
+
             return documents
         except Exception as e:
             print(e)
